package org.robokind.headless;

import java.io.File;
import java.net.URISyntaxException;
import java.util.Map;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.jms.Connection;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MessageConsumer;
import javax.jms.Session;
import org.apache.avro.generic.IndexedRecord;
import org.apache.qpid.client.AMQQueue;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.ServiceRegistration;
import org.robokind.api.common.osgi.OSGiUtils;
import org.robokind.api.common.position.NormalizedDouble;
import org.robokind.api.common.services.ServiceConnectionDirectory;
import org.robokind.api.messaging.RecordPollingService;
import org.robokind.api.motion.Joint;
import org.robokind.api.motion.Robot;
import org.robokind.api.motion.Robot.JointId;
import org.robokind.api.motion.Robot.RobotPositionHashMap;
import org.robokind.api.motion.Robot.RobotPositionMap;
import org.robokind.api.motion.protocol.RobotFrameSource;
import org.robokind.api.motion.servos.ServoController;
import org.robokind.api.motion.servos.ServoRobot;
import org.robokind.api.motion.utils.RobotUtils;
import org.robokind.avrogen.motion.MotionFrameRecord;
import org.robokind.extern.utils.apache_commons_configuration.ConfigUtils;
import org.robokind.impl.messaging.ConnectionManager;
import org.robokind.impl.messaging.JMSPollingService;
import org.robokind.impl.motion.config.RobotConfigXMLReader;
import org.robokind.impl.motion.messaging.JMSRobotServer;

public class Activator implements BundleActivator {
    private final static Logger theLogger = 
            Logger.getLogger(Activator.class.getName());
    
    private final static long theDefaultBlenderInterval = 40L;
    
    @Override
    public void start(BundleContext context) throws Exception {
        Robot robot = startRobot("./resources/robot.xml");
        Thread.sleep(4000);
        Connection connection = ConnectionManager.createConnection(
                "admin", "admin", "client1", "test", "tcp://127.0.0.1:5672");
        String connId = "connection1";
        ServiceRegistration connReg =  ConnectionManager.registerConnection(
                context, connId, connection, null);
        String queue = "test.RobotMoveQueue; {create: always, node: {type: queue}}";
        startRobotServer(context, robot, connId, queue);
    }
    
    private Robot startRobot(String path){
        theLogger.warning("Opening Robot Config File.");
        File file = ConfigUtils.getFileSystemAdapter().openFile(path);
        if(file == null){
            return null;
        }
        BundleContext context = OSGiUtils.getBundleContext(Robot.class);
        if(context == null){
            theLogger.warning(
                    "Unable to load Robot.  Could not find BundleContext.");
            return null;
        }
        theLogger.warning("Loading config and initializing Robot.");
        Robot robot = ServiceConnectionDirectory.buildService(context, 
                ServoRobot.VERSION, RobotConfigXMLReader.VERSION, 
                file, File.class, Robot.class);
        if(robot == null){
            theLogger.warning(
                    "Unable to load Robot.  Could not lind loaders.");
            return null;
        }
        theLogger.log(Level.WARNING, 
                "Connecting Robot: {0}.", robot.getRobotId());
        robot.connect();
        ServiceRegistration reg = 
                RobotUtils.registerRobot(context, robot, null);
        if(reg == null){
            theLogger.log(Level.INFO, "Failed registering Robot service.");
            return null;
        }
        theLogger.log(Level.INFO, "Robot Service Registered.");
        if(robot instanceof ServoRobot){
            setServoRobot(context, (ServoRobot)robot);
        }
        RobotUtils.startDefaultBlender(
                context, robot.getRobotId(), theDefaultBlenderInterval);
        return robot;
    }
    
    private void setServoRobot(BundleContext context, ServoRobot robot){
        ServoRobot sr = (ServoRobot)robot;
        Map<ServoController.Id,ServoController> controllers = 
                sr.getControllers();
        if(controllers == null || controllers.isEmpty()){
            return;
        }
        for(ServoController controller : controllers.values()){
            context.registerService(ServoController.class.getName(), 
                    controller, new Properties());
        }
    }
    
     public static <T extends IndexedRecord> RecordPollingService<T> 
             startPollingService(Class<T> clazz, 
                    BundleContext context, String connectionId, String destStr) 
                            throws JMSException, URISyntaxException, Exception{
         
        ServiceReference ref = ConnectionManager.retrieveConnectionReference(
                context, connectionId, null);
        if(ref == null){
            throw new NullPointerException();
        }
        Object obj = context.getService(ref);
        if(!(obj instanceof Connection)){
            theLogger.warning("Service not expected type.");
            return null;
        }
        Connection connection = (Connection)obj;
        Session session = 
                connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
        Destination dest = new AMQQueue(destStr);
        MessageConsumer moveConsumer = session.createConsumer(dest);
        connection.start();
        RecordPollingService pollService = new JMSPollingService
                (MotionFrameRecord.class, 
                MotionFrameRecord.SCHEMA$, 
                moveConsumer);
        return pollService;
    }
    
    private void startRobotServer(
            BundleContext context, Robot robot, String conStr, String destStr)
                throws Exception{
        Robot.Id id = robot.getRobotId();
        JMSRobotServer server = new JMSRobotServer(context, id);
        RecordPollingService<MotionFrameRecord> poll = startPollingService(
                MotionFrameRecord.class, context, conStr, destStr);
        server.setPollingService(poll);
        poll.start();
    }
    
    public void stop(BundleContext context) throws Exception {
        // TODO add deactivation code here
    }

}
