/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.robokind.impl.messaging;

import java.util.Properties;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import org.apache.qpid.client.AMQConnectionFactory;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;
import org.robokind.api.common.osgi.OSGiUtils;
import org.robokind.api.common.osgi.SingleServiceListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author Matthew Stevenson <www.robokind.org>
 */
public class ConnectionManager {
    private final static Logger theLogger = LoggerFactory.getLogger(ConnectionManager.class);
    private final static String theAMQPFormatString = "amqp://%s:%s@%s/%s?brokerlist='%s'";
    
    public final static String PROP_CONNECTION_ID = "connectionId";
    public final static String PROP_DESTINATION_ID = "destinationId";
    
    public static Connection createConnection(
            String username, String password, 
            String clientName, String virtualHost, String tcpAddress){
        String amqpURL = String.format(theAMQPFormatString, 
                username, password, clientName, virtualHost, tcpAddress);
        return createConnection(amqpURL);
    }
    
    public static Connection createConnection(String amqpURL){
        try{
            ConnectionFactory cf = new AMQConnectionFactory(amqpURL);
            return cf.createConnection();
        }catch(Exception ex){
            theLogger.warn("Unable to create connection.", ex);
            return null;
        }
    }
    
    public static ServiceRegistration registerConnection(BundleContext context, 
            String connectionId, Connection connection, Properties props){
        if(context == null || connection == null){
            throw new NullPointerException();
        }
        return OSGiUtils.registerUniqueService(context, 
                Connection.class.getName(), 
                PROP_CONNECTION_ID, connectionId, 
                connection, props);
    }
    
    public static SingleServiceListener<Connection> buildConnectionTracker(
            BundleContext context, String connectionId, String filter){
        return OSGiUtils.buildSingleServiceListener(
                Connection.class, context, 
                PROP_CONNECTION_ID, connectionId, filter);
    }
    
    public static ServiceRegistration registerDestination(BundleContext context, 
            String destinationId, Destination destination, Properties props){
        if(context == null || destination == null){
            throw new NullPointerException();
        }
        return OSGiUtils.registerUniqueService(context, 
                Destination.class.getName(), 
                PROP_DESTINATION_ID, destinationId, 
                destination, props);
    }
    
    public static SingleServiceListener<Destination> buildDestinationTracker(
            BundleContext context, String destination, String filter){
        return OSGiUtils.buildSingleServiceListener(
                Destination.class, context, 
                PROP_DESTINATION_ID, destination, filter);
    }
}
