/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.robokind.impl.messaging;

import java.util.logging.Logger;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MessageConsumer;
import javax.jms.Session;
import org.apache.avro.Schema;
import org.apache.avro.generic.IndexedRecord;
import org.robokind.api.common.utils.Listener;
import org.robokind.api.messaging.DefaultMessageReceiver;
import org.robokind.api.messaging.RecordReceiver;

/**
 * A DefaultMessageReceiver for receiving Avro Records over JMS (Qpid).
 * @author Matthew Stevenson <www.robokind.org>
 */
public class JMSMessageReceiver<
        Msg, Rec extends IndexedRecord, L extends Listener<Msg>> 
                extends DefaultMessageReceiver<Msg, Rec, L> {
    
    private final static Logger theLogger = 
            Logger.getLogger(JMSMessageReceiver.class.getName());
    private Session mySession;
    private Destination myDestination;
    private Class<Rec> myRecordClass;
    private Schema myRecordSchema;
    
    /**
     * Creates a new JMSMessageReceiver 
     * @param session the JMS Session to use for receiving
     * @param destination the JMS Destination to receive from
     * @param recordClass the Class of the Avro Record being received
     * @param recordSchema the Avro Schema of the Record being received
     */
    public JMSMessageReceiver(Session session, Destination destination, 
            Class<Rec> recordClass, Schema recordSchema){
        if(session == null || destination == null || recordClass == null){
            throw new NullPointerException();
        }
        mySession = session;
        myDestination = destination;
        myRecordClass = recordClass;
        myRecordSchema = recordSchema;
    }
    
    /**
     * Start the JMSMessageReceiver receiving Message.
     * Creates and starts a JMSRecordReceiver to receive Records.
     * @throws JMSException if there is an error creating a JMS MessageConsumer 
     * or starting the JMS Polling Service.
     * @throws Exception inherited throws statement, should not throw Exception
     */
    @Override
    public void start() throws JMSException, Exception{
        MessageConsumer consumer = mySession.createConsumer(myDestination);
        RecordReceiver<Rec> pollingService = new JMSRecordReceiver<Rec>(
                myRecordClass, myRecordSchema, consumer);
        setRecordReceiver(pollingService);
        super.start();
    }
}
