/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.robokind.impl.messaging;

import java.util.logging.Logger;
import javax.jms.Destination;
import javax.jms.Session;
import org.apache.avro.generic.IndexedRecord;
import org.robokind.api.common.utils.Listener;
import org.robokind.api.messaging.DefaultMessageSender;

/**
 * DefaultMessageSender for serializing to Avro Records and sending over 
 * JMS (Qpid).
 * @author Matthew Stevenson <www.robokind.org>
 */
public class JMSMessageSender<Msg, Rec extends IndexedRecord, L extends Listener<Msg>> extends DefaultMessageSender<Msg,Rec,L> {
    private final static Logger theLogger = 
            Logger.getLogger(JMSMessageSender.class.getName());
    private Session mySession;
    private Destination myDestination;
    private Class<Rec> myRecordClass;
    
    /**
     * Creates a new JMSMessageSender
     * @param session JMS Session to use for sending
     * @param destination JMS Destination to send to
     * @param recordClass Class of the Avro Record to send
     */
    public JMSMessageSender(Session session, Destination destination, Class<Rec> recordClass){
        if(session == null || destination == null || recordClass == null){
            throw new NullPointerException();
        }
        mySession = session;
        myDestination = destination;
        myRecordClass = recordClass;
    }
    
    /**
     * Creates and opens a new JMSRecordSender for sending the Avro Records.
     */
    @Override
    public void start(){
        JMSRecordSender<Rec> recordSender = new JMSRecordSender<Rec>(myRecordClass);
        recordSender.setSession(mySession);
        recordSender.setDestination(myDestination);
        recordSender.openProducer();
        setRecordSender(recordSender);
    }
    
    /**
     * Closes the JMSRecordReceiver.
     */
    @Override
    public void stop(){
        if(myRecordSender == null || !(myRecordSender instanceof JMSRecordSender)){
            return;
        }
        ((JMSRecordSender)myRecordSender).closeProducer();
    }
}
