/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.robokind.impl.messaging;

import java.util.logging.Level;
import java.util.logging.Logger;
import javax.jms.BytesMessage;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MessageProducer;
import javax.jms.Session;
import org.apache.avro.generic.IndexedRecord;
import org.robokind.api.messaging.RecordSender;
import org.robokind.bind.apache_qpid.QpidUtils;

/**
 * Sends Avro Records over JMS (Qpid).
 * 
 * @author Matthew Stevenson <www.robokind.org>
 */
public class JMSRecordSender<T extends IndexedRecord> 
        implements RecordSender<T> {
    static final Logger theLogger = Logger.getLogger(JMSRecordSender.class.getName());
    protected Class<T> myClass;
    protected Session mySession;
    protected Destination myDestination;
    protected MessageProducer myMessageProducer;
    
    /**
     * Creates a new JMSRecordSender.
     * @param clazz Class of the Avro Record to send
     */
    public JMSRecordSender(Class<T> clazz){
        if(clazz == null){
            throw new NullPointerException();
        }
        myClass = clazz;
    }
    
    /**
     * Sets the JMS Session to use for sending
     * @param session 
     */
    public void setSession(Session session){
        mySession = session;
    }
    /**
     * Sets the JMS Destination to send to.
     * @param dest JMS Destination to send to
     */
    public void setDestination(Destination  dest){
        myDestination = dest;
    }
    /**
     * Closes the underlying JMS MessageProducer making it unable to send
     * Records.
     */
    public void closeProducer(){
        if(myMessageProducer == null){
            return;
        }
        try{
            myMessageProducer.close();
        }catch(JMSException ex){
            theLogger.log(Level.WARNING, "Error closing MessageProducer.", ex);
        }
    }
    
    /**
     * Opens the underlying JMS MessageProducer, allowing it to begin sending
     * Records.  This must be called before Records can be sent.
     */
    public void openProducer(){
        if(mySession == null || myDestination == null){
            return;
        }
        try{
            myMessageProducer = mySession.createProducer(myDestination);
        }catch(JMSException ex){
            theLogger.log(Level.WARNING, "Error opening MessageProducer.", ex);
        }
    }
    
    /**
     * Serializes the given Avro Record to a byte array, packs it into a JMS
     * BytesMessage, and sends it to the JMS Destination.
     * @param record the Avro Record to send
     */
    @Override
    public void sendRecord(T record){
        if(record == null){
            throw new NullPointerException();
        }
        if(mySession == null || myMessageProducer == null){
            theLogger.log(Level.INFO, 
                    "Not connected, unable to send message.");
            return;
        }
        try{
            BytesMessage msg = mySession.createBytesMessage();
            QpidUtils.packAvroMessage(myClass, record, msg);
            myMessageProducer.send(msg);
        }catch(Exception ex){
            theLogger.log(Level.WARNING, "Error sending message.", ex);
        }
    }    
}
