/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.robokind.impl.messaging;


import java.io.IOException;
import javax.jms.BytesMessage;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Session;
import org.apache.qpid.client.message.JMSBytesMessage;
import org.apache.avro.generic.IndexedRecord;
import org.robokind.api.messaging.RemoteService;
import org.robokind.api.common.playable.AbstractPlayable;
import org.robokind.avrogen.messaging.ServiceCommand;
import org.robokind.avrogen.messaging.ServiceCommandType;
import org.robokind.bind.apache_qpid.QpidUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * A facade for controlling a RemoteService using Avro and JMS (Qpid).
 * A RemoteServiceFacade has a JMS MessageProducer for sending commands to the
 * RemoteService, and a JMS MessageConsumer for receiving error messages.
 * (Error messages are not yet completed)
 * @author Matthew Stevenson <www.robokind.org>
 */
public class RemoteServiceFacade<Config extends IndexedRecord> 
        extends AbstractPlayable implements RemoteService<Config>{
    private final static Logger theLogger = 
            LoggerFactory.getLogger(RemoteServiceFacade.class);
    public final static String CONFIG_MIME_TYPE = "application/config";
    public final static String COMMAND_MIME_TYPE = "application/service-command";
    
    private Class<Config> myConfigClass;
    private Config myReusableConfig;
    private Session mySession;
    private MessageProducer myCommandSender;
    private MessageConsumer myErrorReceiver;
    
    public RemoteServiceFacade(
            Class<Config> configClass, 
            Config reusableConfig,
            Session session, 
            Destination commandDest, 
            Destination errorDest) throws JMSException{
        myConfigClass = configClass;
        myReusableConfig = reusableConfig;
        mySession = session;
        myCommandSender = session.createProducer(commandDest);
        myErrorReceiver = session.createConsumer(errorDest);
    }
    
    public RemoteServiceFacade(
            Class<Config> configClass,
            Config reusableConfig,
            Session session, 
            MessageProducer commandSender, 
            MessageConsumer errorReceiver){
        myConfigClass = configClass;
        myReusableConfig = reusableConfig;
        mySession = session;
        myCommandSender = commandSender;
        myErrorReceiver = errorReceiver;
    }
    
    public void setSession(Session session){
        mySession = session;
    }
    
    public void setSender(MessageProducer producer){
        myCommandSender = producer;
    }
    
    public void setReceiver(MessageConsumer consumer){
        myErrorReceiver = consumer;
    }
    
    @Override
    public void initialize(Config config) throws IOException, JMSException, Exception {
        send(myConfigClass, config, CONFIG_MIME_TYPE);
    }
    
    @Override
    public boolean onStart(long time) {
        return sendCommand(ServiceCommandType.START);
    }

    @Override
    public boolean onPause(long time) {
        return sendCommand(ServiceCommandType.PAUSE);
    }

    @Override
    public boolean onResume(long time) {
        return sendCommand(ServiceCommandType.RESUME);
    }
    
    @Override
    public boolean onComplete(long time){
        return sendCommand(ServiceCommandType.STOP);
    }

    @Override
    public boolean onStop(long time) {
        return sendCommand(ServiceCommandType.STOP);
    }
    
    protected <C extends IndexedRecord> void send(
            Class<C> cls, C c, String contentType) 
            throws IOException, JMSException{
        BytesMessage message = mySession.createBytesMessage();
        ((JMSBytesMessage)message).setContentType(contentType);
        QpidUtils.packAvroMessage(cls, c, message);
        myCommandSender.send(message);
    }
    
    protected boolean sendCommand(ServiceCommandType command){
        ServiceCommand serviceCmd = new ServiceCommand();
        serviceCmd.command = command;
        try{
            send(ServiceCommand.class, serviceCmd, COMMAND_MIME_TYPE);
            return true;
        }catch(Exception ex){
            theLogger.error("Error sending Start Command.", ex);
            return false;
        }
    }
    
    @Override
    public Class<Config> getConfigClass(){
        return myConfigClass;
    }
    
    @Override
    public Config getReusableConfig(){
        return myReusableConfig;
    }
    
}
