/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.robokind.impl.motion.dynamixel;

import org.robokind.impl.motion.dynamixel.enums.ErrorStatus;
import org.robokind.impl.motion.dynamixel.enums.Register;
import org.robokind.impl.motion.dynamixel.enums.Instruction;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.robokind.api.motion.servos.utils.ConnectionStatus;
import org.robokind.impl.motion.serial.SerialServoController;
import org.robokind.api.common.utils.Utils;
import org.robokind.api.common.config.VersionProperty;
import org.robokind.api.common.position.NormalizedDouble;
import org.robokind.api.motion.servos.config.ServoConfig;
/**
 * ServoController for a chain of DynamixelServos.
 * 
 * @author Matthew Stevenson <www.robokind.org>
 */
public class DynamixelController extends SerialServoController<
        DynamixelServo.Id, 
        ServoConfig<DynamixelServo.Id>,
        DynamixelServo,
        DynamixelControllerConfig> {
    private final static Logger theLogger = Logger.getLogger(DynamixelController.class.getName());
    /**
     * Controller type version name.
     */
    public final static String VERSION_NAME = "Dynamixel RX";
    /**
     * Controller type version number.
     */
    public final static String VERSION_NUMBER = "1.0";
    /**
     * Controller type VersionProperty.
     */
    public final static VersionProperty VERSION = new VersionProperty(VERSION_NAME, VERSION_NUMBER);
    /**
     * DynamixelController broadcast id.  This is an invalid DynamixelServo 
     * physical id, reserved to indicate the controller to the joints.
     */
    public final static DynamixelServo.Id BROADCAST_ID = 
            new DynamixelServo.Id();

    private Map<DynamixelServo.Id,ServoConfig> myUnresponsiveServos;
    private Boolean myEnabledFlag;

    /**
     * Creates a new DynamixelController from the given SerialServoControllerConfig.
     * @param config SerialServoControllerConfig for the new controller
     */
    public DynamixelController(DynamixelControllerConfig config){
        super(config);
        myEnabledFlag = true;
        setTimeoutLength(20);
    }

    /**
     * Attempts to ping each Servo in the ServoControllerConfig.  A joint is 
     * added if it responds, otherwise it is added to the UnresponsiveServos 
     * List.
     * @return true if successful
     */
    @Override
    protected synchronized boolean setServos(){
        myServos.clear();
        myServoMap.clear();
        myUnresponsiveServos = new HashMap();
        for(ServoConfig<DynamixelServo.Id> param : myConfig.getServoConfigs().values()){
            DynamixelServo.Id id = param.getServoId();
            if(!ping(id)){
                myUnresponsiveServos.put(param.getServoId(), param);
                theLogger.log(Level.WARNING,
                        "Unable to ping Dynamixel {0}.",
                        new Object[]{param.getServoId()});
                continue;
            }
            DynamixelServo servo = new DynamixelServo(param, this);
            myServos.add(servo);
            DynamixelServo.Id sId = servo.getId();
            ServoId<DynamixelServo.Id> servoId = 
                    new ServoId<DynamixelServo.Id>(getId(), sId);
            myServoMap.put(servoId, servo);
            initServo(servo);
        }
        ServoId[] ids = myServoMap.keySet().toArray(new ServoId[0]);
        return moveServosSync(3000, ids);
    }
    
    private void initServo(DynamixelServo servo){
        servo.setEnabled(myEnabledFlag);
        NormalizedDouble def = servo.getDefaultPosition();
        servo.setGoalPosition(def);
    }

    @Override
    public boolean moveServo(ServoId<DynamixelServo.Id> id, long lenMillisec) {
        return moveServos(new ServoId[]{id}, 1, 0, lenMillisec);
    }

    /**
     * Number of bytes sent to each DynamixelServo when moving multiple Servos.
     */
    private final static int theSyncCount = 5;
    
    @Override
    public boolean moveServos(ServoId<DynamixelServo.Id>[] ids, 
    int len, int offset, long lenMillisec) {
        if(ConnectionStatus.CONNECTED != myConnectionStatus){
            return false;
        }
        return moveServosSync(lenMillisec, ids);
    }
    
    private synchronized boolean moveServosSync(long lenMillisec, ServoId<DynamixelServo.Id>... ids) {
        Boolean e = getEnabled();
        if(e == null || !e){
            return true;
        }
        List<Byte> data = new ArrayList<Byte>(ids.length*theSyncCount);
        for(ServoId<DynamixelServo.Id> id : ids){
            DynamixelServo servo = myServoMap.get(id);
            if(servo == null || !servo.getEnabled() || !servo.changed()){
                continue;
            }
            Integer goal = servo.getAbsoluteGoalPosition();
            if(goal == null){
                continue;
            }
            lenMillisec = Math.max(lenMillisec, 1);
            int speed = calculateSpeed(servo, goal, lenMillisec);
            data.add((byte)servo.getPhysicalId().getIntValue());
            data.add((byte)(goal & 0xff));
            data.add((byte)(goal >> 8));
            data.add((byte)(speed & 0xff));
            data.add((byte)(speed >> 8));
            servo.setPreviousPosition(goal);
        }
		if(data.isEmpty()){
            return true;
        }
        return syncWrite(Register.GoalPosition, data.size()/theSyncCount, data);
    }
    
    private int calculateSpeed(DynamixelServo servo, int goal, long time){
        int cur = servo.getPreviousPosition();
        int dist = Math.abs(goal - cur);
        double theta = dist*300.0/1023.0;
        time = Math.max(time, 1);
        
        //60000ms/minute, 360 deg/rotation
        double rpm = theta * (60000.0/time) / 360.0; 
        double speed = rpm/0.111;
        return Utils.bound((int)speed,1,1023);
    }

    @Override
    public boolean moveAllServos(long lenMillisec) {
        ServoId[] ids = myServoMap.keySet().toArray(new ServoId[0]);
        return moveServos(ids, ids.length, 0, lenMillisec);
    }
    
    /**
     * Reads a Packet from the DynamixelServo chain.
     * @return Packet read from the DynamixelServo chain
     */
    protected synchronized Packet readPacket(){
        Packet packet = new Packet();
        if(myPort.read() != 0xff || myPort.read() != 0xff){
            theLogger.log(Level.WARNING, "Error reading Dynamixel Packet.  Incorrect header bytes.");
            packet.error = true;
            return packet;
        }
        int receivedId = myPort.read();
        packet.length = myPort.read()-2;
        int error = myPort.read();
        packet.errors = ErrorStatus.getStatusList(error);
        packet.data = myPort.read(packet.length);
        packet.error = (packet.length != packet.data.length);
        byte chkCalc = Utils.checksum(packet.data, 0, packet.data.length, true,
                (byte)receivedId, (byte)(packet.length+2),(byte)error);
        byte chkRec = (byte)myPort.read();
        boolean chksum = chkCalc != chkRec;
        boolean dataLen = packet.length != packet.data.length;
        boolean errorFlag = !packet.errors.isEmpty();
        boolean badId = !DynamixelServo.Id.isValidId(receivedId);
        if(!badId){
            packet.id = new DynamixelServo.Id(receivedId);
        }
        packet.error = chksum || dataLen || errorFlag || badId;
        if(!packet.error){
            return packet;
        }else if(chksum){
            theLogger.log(Level.WARNING, "Error reading Dynamixel packet. Bad checksum.");
        }if (dataLen){
            theLogger.log(Level.WARNING, "Error reading Dynamixel packet.  Data not recieved.");
        }if (errorFlag){
            int len = packet.errors.size();
            String errorStr = "";
            for(int i=0; i<len; i++){
                ErrorStatus e = packet.errors.get(i);
                errorStr += e.name();
                errorStr += i < len-1 ? ", " : ".";
            }
            theLogger.log(Level.WARNING, "Error reading Dynamixel packet.  {0}", errorStr);
        }if (badId){
            theLogger.log(Level.WARNING, "Error reading Dynamixel packet.  Invalid Id Recieved.");
        }
        return packet;
	}

    /**
     * Sends the given Instruction with parameters to the Servo at the given 
     * physical id.
     * @param id physical id of the Servo
     * @param ins Instruction to send
     * @param params Instruction parameters
     * @return true if the write is successful
     */
    protected synchronized boolean writeInstruction(DynamixelServo.Id id, Instruction ins, byte... params){
        int paramLen = params == null ? 0 : params.length;
        byte[] data = new byte[6 + paramLen];
        data[0] = (byte)0xff;
        data[1] = (byte)0xff;
        data[2] = (byte)id.getIntValue();
        data[3] = (byte)(paramLen + 2);
        data[4] = ins.getByte();
        System.arraycopy(params, 0, data, 5, paramLen);
		data[data.length - 1] = Utils.checksum(data, 2, data.length-1, true);
		return myPort.write(data) && myPort.flushWriter();
	}

    /**
     * Pings the DynamixelServo at the given physical id.
     * @param id physical id of the DynamixelServo to ping
     * @return true if the DynamixelServo responds
     */
    public synchronized boolean ping(DynamixelServo.Id id) {
		writeInstruction(id, Instruction.Ping);
        Packet packet = readPacket();
        return (!packet.error && packet.id.equals(id));
	}
    
    /**
     * Reads the Register value from the DynamixelServo with the given physical 
     * id.
     * @param id physical id of the DynamixelServo to read
     * @param reg Register to read
     * @return unsigned value at the given register
     */
    public int readRegister(DynamixelServo.Id id, Register reg){
		return readRegisters(id, reg, reg)[0];
	}

    /**
     * Reads the values from the Register range specified by regFirst and 
     * regLast for the DynamixelServo with the given physical id.
     * @param id physical id of the DynamixelServo to read
     * @param regFirst first Register in the range
     * @param regLast last Register in the range
     * @return array of unsigned values from the Registers specified
     */
    public synchronized int[] readRegisters(DynamixelServo.Id id, Register regFirst, Register regLast){
		byte byteCount = (byte)(regLast.getByte() - regFirst.getByte() + regLast.getLength());
		Register[] regs = Register.values();
		int first = regFirst.ordinal();
		int last = regLast.ordinal();
		int[] values = new int[last - first + 1];
		writeInstruction(id, Instruction.ReadData, regFirst.getByte(), byteCount);
        Packet packet = readPacket();
        
		for (int i = first; i <= last; i++){
			Register reg = regs[i];
			int offset = reg.getByte() - regFirst.getByte();
			values[i - first] = Utils.unsign(packet.data[offset]);
            if(reg.getLength() > 1){
				values[i - first] += (Utils.unsign(packet.data[offset + 1]) << 8);
            }
		}
		return values;
	}

    /**
     * Write a value to the given register for the given DynamixelServo.
     * @param id physical id of the DynamixelServo
     * @param reg Register to write
     * @param value value to write
     * @param deferred if true, data is written but the DynamixelServo state 
     * will not change until it receives an ACTION instruction.
     * @return true if successful
     */
    protected boolean writeRegister(DynamixelServo.Id id, Register reg, Integer value, Boolean deferred){
        byte[] params = new byte[reg.getLength() + 1];
        params[0] = reg.getByte();
        if(reg.getLength() == 1){
            params[1] = (byte) value.byteValue();
        }else{
            params[1] = (byte) (value & 0xff);
            params[2] = (byte) (value >> 8);
        }
		boolean write = writeInstruction(id, deferred ? Instruction.RegWrite : Instruction.WriteData, params);
        if(!write){
            return false;
        }else if (!deferred){
			return readPacket().error;
        }
        return true;
	}

    /**
     * Write data to a range of Registers for multiple DynamixelServos 
     * simultaneously.
     * @param startAddress the first Register to write for each Servo
     * @param jointCount number of DynamixelServos to write to
     * @param params an List of bytes.  For each DynamixelServo, params contains
     * the physical ids and Register parameters for each Register to write to
     * @return true if successful
     */
    protected boolean syncWrite(Register startAddress, int jointCount, List<Byte> params){
        int paramLen = params.size();
		if((paramLen % jointCount) != 0){
			throw new IllegalArgumentException("Dyanmixel SyncWrite params length() error");
        }
        byte[] data = new byte[paramLen + 2];
        data[0] = startAddress.getByte();
        data[1] = (byte)(paramLen/jointCount - 1);
        for(int i=0; i<paramLen; i++){
            data[i+2] = params.get(i);
        }
		return writeInstruction(BROADCAST_ID, Instruction.SyncWrite, data);
	}

    @Override
    protected DynamixelServo connectServo(ServoConfig config) {
        return new DynamixelServo(config, this);
    }

    @Override
    protected boolean disconnectServo(ServoId<DynamixelServo.Id> id) {
        return true;
    }
    
    @Override
    public void setEnabled(Boolean enabled){
        Boolean old = myEnabledFlag;
        myEnabledFlag = enabled;
        firePropertyChange(PROP_ENABLED, old, enabled);
    }

    @Override
    public Boolean getEnabled(){
        return myEnabledFlag;
    }

    @Override
    public Class<DynamixelServo.Id> getServoIdClass() {
        return DynamixelServo.Id.class;
    }

    /**
     * Data packet returned from DynamixelServos.
     */
    protected class Packet{
        DynamixelServo.Id id = BROADCAST_ID;
        byte[] data = null;
        int length;
        boolean error = false;
        List<ErrorStatus> errors = null;
    }
}