/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.robokind.integration.animation_motion;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.logging.Logger;
import org.robokind.api.animation.Animation;
import org.robokind.api.animation.Channel;
import org.robokind.api.animation.compiled.CompiledMap;
import org.robokind.api.animation.compiled.CompiledPath;
import org.robokind.api.animation.player.AnimationJob;
import org.robokind.api.animation.player.AnimationJobListener;
import org.robokind.api.animation.player.AnimationPlayer;
import org.robokind.api.common.playable.PlayState;
import org.robokind.api.common.playable.AbstractPlayable;
import org.robokind.api.common.position.NormalizedDouble;
import org.robokind.api.motion.Robot;
import org.robokind.api.motion.Robot.RobotPositionMap;
import org.robokind.api.motion.Joint;
import org.robokind.api.motion.protocol.MotionFrame;
import org.robokind.api.motion.protocol.FrameSource;
import org.robokind.api.motion.protocol.DefaultMotionFrame;

/**
 * An implementation of an AnimationJob which expects to be queried for new
 * positions as a FrameSource.
 * 
 * @author Matthew Stevenson <www.robokind.org>
 */
public class AnimationJobFrameSource extends AbstractPlayable implements 
        AnimationJob, FrameSource<RobotPositionMap> {
    private final static Logger theLogger = Logger.getLogger(AnimationJobFrameSource.class.getName());
    private RobotPositionMap myPreviousPositions;
    private Animation myAnimation;
    private CompiledMap myAnimationMap;
    private List<AnimationJobListener> myAnimationListeners;
    private AnimationPlayer mySource;
    private Robot.Id myRobotId;
    private boolean myLoopFlag;

    AnimationJobFrameSource(AnimationPlayer source, Robot.Id robotId, 
            Animation anim, long stepLength, Long start, Long stop){
        if(robotId == null){
            throw new NullPointerException();
        }if(anim == null){
            throw new NullPointerException();
        }if(stepLength <= 0){
            throw new IllegalArgumentException("stepLength must be greater than zero");
        }
        myLoopFlag = false;
        myRobotId = robotId;
        myAnimationListeners = new ArrayList();
        mySource = source;
        myAnimation = anim;
        long s = start == null ? -1 : start;
        long e = stop == null ? -1 : stop;
        myAnimationMap = anim.compileMap(s, e, stepLength);
        List<Channel> channels = myAnimation.getChannels();
        myPreviousPositions = new Robot.RobotPositionHashMap(channels.size());
        for(Channel c : channels){
            if(c == null){
                continue;
            }
            CompiledPath path = myAnimationMap.get(c.getId());
            if(path == null){
                continue;
            }
            Integer id = c.getId();
            double v = path.estimatePosition(0);
            if(id == null || !NormalizedDouble.isValid(v)){
                continue;
            }
            NormalizedDouble val = new NormalizedDouble(v);
            Joint.Id jId = new Joint.Id(id);
            Robot.JointId djId = new Robot.JointId(myRobotId, jId);
            myPreviousPositions.put(djId, val);
        }        
    }

    /**
     * Sets the AnimationPlayer which created the AnimationJobFrameSource.
     * @param source AnimationPlayer which created the AnimationJobFrameSource
     */
    public void setSource(AnimationPlayer source){
        mySource = source;
    }
    
    @Override
    public AnimationPlayer getSource() {
        return mySource;
    }
    
    @Override
    public void addAnimationListener(AnimationJobListener listener) {
        if(myAnimationListeners == null){
            myAnimationListeners = new ArrayList<AnimationJobListener>();
        }
        if(!myAnimationListeners.contains(listener)){
            myAnimationListeners.add(listener);
        }
    }
    
    @Override
    public void removeAnimationListener(AnimationJobListener listener) {
        if(myAnimationListeners == null){
            return;
        }
        myAnimationListeners.remove(listener);
    }
    
    @Override
    public Animation getAnimation(){
        return myAnimation;
    }
    
    public Long getCurrentTime(long time){
        Long l = getElapsedPlayTime(time);
        if(l == null){
            return null;
        }
        Long s = myAnimationMap.getStartTime();
        if(s == null){
            return l;
        }
        return l+s;
    }
    
    @Override
    public Map<Integer,Double> advanceAnimation(long time, long interval){
        MotionFrame frame = getMovements(time, interval);
        if(frame == null){
            return null;
        }
        return frame.getGoalPositions();
    }
    
    @Override
    public MotionFrame getMovements(long time, long interval) {
        if(myPlayState != PlayState.RUNNING){
            return null;
        }
        MotionFrame frame = new DefaultMotionFrame();
        frame.setTimestampMillisecUTC(time);
        frame.setFrameLengthMillisec(interval);
        myPreviousPositions.clear();
        long cur = getCurrentTime(time);
        for(Entry<Integer,CompiledPath> e : myAnimationMap.entrySet()){
            CompiledPath path = e.getValue();
            Integer id = e.getKey();
            double pos = path.estimatePosition(cur);
            if(id == null || !NormalizedDouble.isValid(pos)){
                continue;
            }
            NormalizedDouble val = new NormalizedDouble(pos);
            Joint.Id jId = new Joint.Id(id);
            Robot.JointId djId = 
                    new Robot.JointId(myRobotId, jId);
            myPreviousPositions.put(djId, val);
        }
        frame.setPreviousPositions(myPreviousPositions);
        myPreviousPositions.clear();
        cur = getCurrentTime(time+interval);
        boolean done = true;
        for(Entry<Integer,CompiledPath> e : myAnimationMap.entrySet()){
            CompiledPath path = e.getValue();
            Integer id = e.getKey();
            double pos = path.estimatePosition(cur);
            
            done = done && (cur >= path.getEndTime());
            if(id == null || !NormalizedDouble.isValid(pos)){
                continue;
            }
            NormalizedDouble val = new NormalizedDouble(pos);
            Joint.Id jId = new Joint.Id(id);
            Robot.JointId djId = 
                    new Robot.JointId(myRobotId, jId);
            myPreviousPositions.put(djId, val);
        }
        frame.setGoalPositions(myPreviousPositions);
        if(done){
            complete(time);
        }
        advance(time);
        return frame;
    }

    private void advance(long time){
        if(myAnimationListeners == null){
            return;
        }
        for(AnimationJobListener listener : myAnimationListeners){
            listener.animationAdvanced(getCurrentTime(time));
        }
    }

    @Override
    public boolean onStart(long time) {
        if(myAnimationListeners == null){
            return true;
        }
        for(AnimationJobListener listener : myAnimationListeners){
            listener.animationStart(myAnimationMap.getStartTime(), myAnimationMap.getEndTime());
        }
        return true;
    }
    
    @Override
    public Long getAnimationLength(){
        return myAnimationMap.getEndTime() - myAnimationMap.getStartTime();
    }
    
    @Override
    public Long getRemainingTime(long time){
        long now = getElapsedPlayTime(time);
        long end = getAnimationLength();
        return end - now;
    }

    @Override
    protected boolean onPause(long time) {
        return true;
    }
    @Override
    protected boolean onResume(long time) {
        return true;
    }
    @Override
    protected boolean onStop(long time) {
        return true;
    }
    @Override
    protected boolean onComplete(long time) {
        return true;
    }
    
    @Override
    protected void afterComplete(long time) {
        if(myLoopFlag){
            start(time);
        }
    }

    @Override
    public void setLoop(boolean loop) {
        myLoopFlag = loop;
    }

    @Override
    public boolean getLoop() {
        return myLoopFlag;
    }
}
