/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.robokind.integration.animation_motion;

import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;
import org.robokind.api.animation.player.AnimationPlayer;
import org.robokind.api.animation.utils.AnimationEditListener;
import org.robokind.api.animation.utils.ChannelsParameterSource;
import org.robokind.api.animation.utils.PositionAdder;
import org.robokind.api.common.osgi.OSGiUtils;
import org.robokind.api.motion.Robot;
import org.robokind.api.motion.protocol.FrameSource;
import org.robokind.api.motion.utils.RobotUtils;

/**
 *
 * @author Matthew Stevenson <www.robokind.org>
 */
public class AnimationMotionUtils {
    private final static Logger theLogger = Logger.getLogger(AnimationMotionUtils.class.getName());
    
    public final static long DEFAULT_BLENDER_INTERVAL = 40L;
    
    public static ServiceRegistration registerRobotAnimationPlayer(
            BundleContext context, Robot.Id robotId){
        Boolean exists = OSGiUtils.serviceExists(
                context, AnimationPlayer.class.getName(),
                RobotUtils.getRobotFilter(robotId));
        if(exists == null || exists){
            return null;
        }
        return registerAnimationPlayer(context, robotId);
    }
    
    public static ServiceRegistration[] registerAnimationEditingComponents(
            BundleContext context, Robot.Id robotId){
        Class[] classes = new Class[]{
                ChannelsParameterSource.class, 
                AnimationEditListener.class, 
                PositionAdder.class};
        Boolean exists = OSGiUtils.serviceExists(context, classes, null);
        if(exists == null || exists){
            return null;
        }
        boolean player = OSGiUtils.serviceExists(
                context, AnimationPlayer.class, null);
        int n = player ? 3 : 4;
        ServiceRegistration[] regs = new ServiceRegistration[n];
        regs[0] = registerChannelRobotParams(context, robotId);
        regs[1] = registerAnimationEditListener(context, robotId);
        regs[2] = registerJointSnapshotAdder(context, robotId);
        if(!player){
            regs[3] = registerAnimationPlayer(context, robotId);
        }
        return regs;
    }
    
    private static ServiceRegistration registerAnimationPlayer(
            BundleContext context, Robot.Id robotId){
        AnimationPlayer player = new DefaultAnimationPlayer(context, robotId);
        Properties props = new Properties();
        props.put(Robot.PROP_ID, robotId.toString());
        ServiceRegistration reg = context.registerService(
                AnimationPlayer.class.getName(), player, props);
        theLogger.log(Level.INFO, 
                "OSGiAnimationPlayer Service Registered Successfully.");
        return reg;
    }
    
    private static ServiceRegistration registerChannelRobotParams(
            BundleContext context, Robot.Id robotId){
        ChannelRobotParameters cdp = 
                new ChannelRobotParameters(context, robotId);
        Properties props = new Properties();
        props.put(Robot.PROP_ID, robotId.toString());
        ServiceRegistration reg = context.registerService(
                ChannelsParameterSource.class.getName(), cdp, props);
        theLogger.log(Level.INFO, 
                "ChannelRobotParameters Service Registered Successfully.");
        return reg;
    }
    
    private static ServiceRegistration registerAnimationEditListener(
            BundleContext context, Robot.Id robotId){
        AnimationEditFrameSource aefs = 
                new AnimationEditFrameSource(context, robotId);
        String[] classes = {
            AnimationEditListener.class.getName(), 
            FrameSource.class.getName()};
        Properties props = new Properties();
        props.put(Robot.PROP_ID, robotId.toString());
        
        ServiceRegistration reg = context.registerService(classes, aefs, props);
        theLogger.log(Level.INFO, 
                "AnimationEditFrameSource Service Registered Successfully.");
        return reg;
    }
    
    private static ServiceRegistration registerJointSnapshotAdder(
            BundleContext context, Robot.Id robotId){
        JointSnapshot js = new JointSnapshot(context, robotId);
        Properties props = new Properties();
        props.put(Robot.PROP_ID, robotId.toString());
        ServiceRegistration reg = context.registerService(
                PositionAdder.class.getName(), js, props);
        theLogger.log(Level.INFO, 
                "JointSnapshot Service Registered Successfully.");
        return reg;
    }
}
