/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.robokind.integration.animation_motion;

import java.awt.geom.Point2D;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.osgi.framework.BundleContext;
import org.robokind.api.animation.editor.AnimationEditor;
import org.robokind.api.animation.editor.ChannelEditor;
import org.robokind.api.animation.editor.EditState;
import org.robokind.api.animation.editor.MotionPathEditor;
import org.robokind.api.animation.utils.PositionAdder;
import org.robokind.api.motion.Robot;
import org.robokind.api.motion.Joint;
import org.robokind.api.common.osgi.ClassTracker;
import org.robokind.api.common.position.NormalizedDouble;
import org.robokind.api.motion.JointProperty;
import org.robokind.api.motion.joint_properties.ReadCurrentPosition;

/**
 *
 * @author Matthew Stevenson <www.robokind.org>
 */
public class JointSnapshot implements PositionAdder{
    private ClassTracker<Robot> myRobotTracker;
    private Robot.Id myRobotId;
    private BundleContext myContext;
    
    public JointSnapshot(BundleContext context, Robot.Id robotId){
        if(context == null || robotId == null){
            throw new NullPointerException();
        }
        myContext = context;
        myRobotId = robotId;
    }
    
    private ClassTracker<Robot> getRobotTracker(){
        if(myRobotTracker != null){
            return myRobotTracker;
        }
        myRobotTracker = new ClassTracker<Robot>(myContext, Robot.class.getName(),null,null);
        return myRobotTracker;
    }
    
    public Robot getRobot(){
        ClassTracker<Robot> tracker = getRobotTracker();
        return tracker == null ? null : tracker.getTopService();
    }
    
    private Map<Integer,Double> getRobotSnapshot(){
        Robot robot = getRobot();
        if(robot == null){
            return Collections.EMPTY_MAP;
        }
        Map<Integer,Double> myPositions = new HashMap<Integer, Double>();
        for(Joint joint : robot.getJoints().values()){
            Joint.Id jId = joint.getId();
            int id = jId.getLogicalJointNumber();
            if(myPositions.containsKey(id)){
                continue;
            }
            JointProperty<NormalizedDouble> prop = joint.getProperty(
                    ReadCurrentPosition.PROPERTY_NAME, NormalizedDouble.class);
            if(prop == null){
                continue;
            }
            NormalizedDouble val = prop.getValue();
            if(val == null){
                continue;
            }
            double pos = val.getValue();
            myPositions.put(id, pos);
        }
        return myPositions;
    }
    
    private Map<Integer,Double> getRobotSnapshot(int...ids){
        Robot robot = getRobot();
        if(robot == null || ids.length == 0){
            return Collections.EMPTY_MAP;
        }
        Map<Integer,Double> myPositions = new HashMap<Integer, Double>();
        for(int id : ids){
            Joint.Id jId = new Joint.Id(id);
            Robot.JointId djId =
                    new Robot.JointId(myRobotId, jId);
            Joint joint = robot.getJoint(djId);
            if(myPositions.containsKey(id) || joint == null){
                continue;
            }
            JointProperty<NormalizedDouble> prop = joint.getProperty(
                    ReadCurrentPosition.PROPERTY_NAME, NormalizedDouble.class);
            if(prop == null){
                continue;
            }
            NormalizedDouble val = prop.getValue();
            if(val == null){
                continue;
            }
            double pos = val.getValue();
            myPositions.put(id, pos);
        }
        return myPositions;
    }
    
    @Override
    public void addPositions(AnimationEditor editor, double x){
        Map<Integer,Double> positions = getRobotSnapshot();
        addSnapshot(editor, x, positions);
    }
    
    public void addSnapshot(AnimationEditor editor, double x, int...ids){
        Map<Integer,Double> positions = getRobotSnapshot(ids);
        addSnapshot(editor, x, positions);
    }
    
    private void addSnapshot(AnimationEditor editor, double x, Map<Integer,Double> positions){
        if(editor == null){
            return;
        }
        List<ChannelEditor> chans = editor.getChildren();
        if(chans == null || chans.isEmpty()){
            return;
        }
        for(ChannelEditor chan : chans){
            int id = chan.getId();
            Double pos = positions.get(id);
            if(pos == null){
                continue;
            }
            MotionPathEditor path = getBestPath(chan, x);
            if(path == null){
                continue;
            }
            path.addChild(this, new Point2D.Double(x, pos));
        }
    }
    
    private MotionPathEditor getBestPath(ChannelEditor chan, double x){
        if(chan.hasFlag(EditState.SELECTED)){
            MotionPathEditor path = chan.getSelected();
            if(path != null){
                return path;
            }
        }
        List<MotionPathEditor> paths = chan.getChildren();
        double bestDist = Double.MAX_VALUE;
        MotionPathEditor bestPath = null;
        for(MotionPathEditor path : paths){
            double start = path.getStart();
            double end = path.getEnd();
            if(x >= start && x <= end){
                return path;
            }
            double dist = (x < start) ? start-x : x-end;
            if(dist < bestDist){
                bestDist = dist;
                bestPath = path;
            }
        }
        return bestPath;
    }
}
